@@ -11,17 +11,20 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 # OTHER DEALINGS IN THE SOFTWARE.
 #####################################################################
-
 """
 OOP implementation of bencode decoders and encoders.
 
 This style is not recommended as it can get bulky. The json-like api
 from the bencode.py module is much easier to use.
+
+Classes
+-------
+* Bendecoder
+* Benencoder
 """
 
 import os
 import re
-import json
 from pyben.exceptions import FilePathError, DecodeError, EncodeError
 
 
@@ -32,14 +35,14 @@ class Bendecoder:
         """
         Initialize instance with optional pre compiled data.
 
-        Args
-        --------
-        data: bytes-like
-            (Optional) (default=None)Target data for decoding.
+        Args:
+        ----
+        data : `bytes` or `bytearray`
+            (Optional) (default=None) Target data for decoding.
+
         """
         self.data = data
         self.decoded = None
-        self.json = None
 
     @classmethod
     def load(cls, item):
@@ -47,29 +50,32 @@ class Bendecoder:
         Extract contents from path/path-like and return Decoded data.
 
         Args
-        --------
-        path : path-like or iobuffer
+        ----
+        path : `str` or `path-like`
             Path containing bencoded data.
 
         Raises
-        ---------
-        FilePathError:
+        ------
+        `FilePathError`:
             Incorrect path or IOBuffer doesnt exist.
 
         Returns
-        ---------
+        -------
         any
             Decoded contents of file, Usually a dictionary.
+
         """
         decoder = cls()
         if hasattr(item, "read"):
             data = item.read()
             return decoder.decode(data)
-        elif os.path.exists(item) and os.path.isfile(item):
-            with open(item, "rb") as fd:
-                data = fd.read()
+
+        if os.path.exists(item) and os.path.isfile(item):
+            with open(item, "rb") as _fd:
+                data = _fd.read()
                 return decoder.decode(data)
-        raise FilePathError()
+
+        raise FilePathError(item)
 
     @classmethod
     def loads(cls, data):
@@ -77,14 +83,15 @@ class Bendecoder:
         Shortcut to Decode raw bencoded data.
 
         Args
-        --------
-        data : bytes
-            Bendencoded bytes.
+        ----
+        data : ``bytes`` or `bytearray`
+            Bendencoded `bytes`.
 
         Returns
-        ---------
+        -------
         any
             Decoded data usually a dictionary.
+
         """
         decoder = cls()
         return decoder.decode(data)
@@ -94,151 +101,168 @@ class Bendecoder:
         Decode bencoded data.
 
         Args
-        -------
-        bits : bytes
+        ----
+        bits : ``bytes``
             bencoded data for decoding.
 
         Returns
-        --------
-        any
+        -------
+        any :
             the decoded data.
+
         """
         data = self.data if not data else data
         self.decoded, _ = self._decode(bits=data)
         return self.decoded
 
-    def json(self, bits=None):
-        """Return output of the decoder json serialized."""
-        if not bits:
-            if self.decoded is None:
-                self.decoded = self.decode(self.data)
-        else:
-            self.decoded = self.decode(bits)
-        self.json = json.dumps(self.decoded)
-        return self.json
-
     def _decode(self, bits=None):
+        """
+        Decode bencoded data.
+
+        Args
+        ----
+        bits : ``bytes``
+            Bencoded data for decoding.
+
+        Returns
+        -------
+        any :
+            The decoded data.
+
+        """
         if bits is None:
             bits = self.data
+
         if bits.startswith(b"i"):
             match, feed = self._decode_int(bits)
             return match, feed
 
         # decode string
-        elif chr(bits[0]).isdigit():
+        if chr(bits[0]).isdigit():
             num, feed = self._decode_str(bits)
             return num, feed
 
         # decode list and contents
-        elif bits.startswith(b"l"):
+        if bits.startswith(b"l"):
             lst, feed = self._decode_list(bits)
             return lst, feed
 
         # decode dictionary and contents
-        elif bits.startswith(b"d"):
+        if bits.startswith(b"d"):
             dic, feed = self._decode_dict(bits)
             return dic, feed
-        else:
-            raise DecodeError(bits, "Unrecognized data. Cannot decode.")
+
+        raise DecodeError(bits)
 
     def _decode_dict(self, bits):
         """
         Decode keys and values in dictionary.
 
         Args
-        --------
-        bits: bytearray
-            Bytes of data for decoding.
+        ----
+        bits : ``bytes`` or `bytearray`
+            `Bytes` of data for decoding.
 
         Returns
-        ---------
-        dict
+        -------
+        `dict` :
             Dictionary and contents.
+
         """
-        dic, feed = {}, 1
+        dct, feed = {}, 1
         while not bits[feed:].startswith(b"e"):
             match1, rest = self._decode(bits[feed:])
             feed += rest
             match2, rest = self._decode(bits[feed:])
             feed += rest
-            dic[match1] = match2
+            dct[match1] = match2
         feed += 1
-        return dic, feed
+        return dct, feed
 
-    def _decode_list(self, bits):
+    def _decode_list(self, data):
         """
         Decode list and its contents.
 
         Args
-        --------
-        bits : bytearray
+        ----
+        bits : `bytearray`
             Bencoded data.
 
         Returns
-        ---------
-        list:
+        -------
+        `list`:
             decoded list and contents
+
         """
-        lst, feed = [], 1
-        while not bits[feed:].startswith(b"e"):
-            match, rest = self._decode(bits[feed:])
-            lst.append(match)
+        seq, feed = [], 1
+        while not data[feed:].startswith(b"e"):
+            match, rest = self._decode(data[feed:])
+            seq.append(match)
             feed += rest
         feed += 1
-        return lst, feed
+        return seq, feed
 
-    def _decode_str(self, bits):
+    @staticmethod
+    def _decode_str(bits):
         """
         Decode string.
 
         Args
-        --------
-        bits: bytes-like
+        ----
+        bits : ``bytes`` or `bytearray`
             Bencoded string.
 
         Returns
-        ---------
-        str:
+        -------
+        `str`:
             Decoded string.
+
         """
         match = re.match(br"(\d+):", bits)
-        word_len, start = int(match.groups()[0]), match.span()[1]
-        word = bits[start : start + word_len]
+        word_size, start = int(match.groups()[0]), match.span()[1]
+        finish = start + word_size
+        word = bits[start:finish]
+
         try:
             word = word.decode("utf-8")
-        except:
-            word = word.hex()
-        return word, start + word_len
 
-    def _decode_int(self, bits):
+        except UnicodeDecodeError:
+            pass
+
+        return word, finish
+
+    @staticmethod
+    def _decode_int(bits):
         """
         Decode integer type.
 
         Args
-        --------
-        bits: bytes-like
+        ----
+        bits : ``bytes`` or `bytearray`
             Bencoded intiger.
 
         Returns
-        ---------
-        int:
+        -------
+        `int`:
             Decoded intiger.
+
         """
         obj = re.match(br"i(-?\d+)e", bits)
         return int(obj.group(1)), obj.end()
 
 
 class Benencoder:
-    """Encode collection of methods for Bencoding data."""
+    """Encoder for bencode encoding used for Bittorrent meta-files."""
 
     def __init__(self, data=None):
         """
         Initialize Benencoder insance with optional pre compiled data.
 
-        Args
-        --------
+        Args:
+        ----
         data : any
             (Optional) Target data for encoding. Defaults to None.
+
         """
         self.data = data
         self.encoded = None
@@ -249,23 +273,23 @@ class Benencoder:
         Shortcut Classmethod for encoding data and writing to file.
 
         Args
-        --------
+        ----
         data : any
             Raw data to be encoded, usually dict.txt
-        path : path-like or iobuffer
+        path : path-like or `BytesIO`
             Where encoded data should be written to.py
 
         Returns
-        ---------
-        bool:
-             Return True if success.txt
+        -------
+        `bool` : Return True if success.txt
+
         """
         encoded = cls(data).encode()
         if hasattr(path, "write"):
             path.write(encoded)
         else:
-            with open(path, "wb") as fd:
-                fd.write(encoded)
+            with open(path, "wb") as _fd:
+                _fd.write(encoded)
         return True
 
     @classmethod
@@ -274,14 +298,14 @@ class Benencoder:
         Shortcut method for encoding data and immediately returning it.
 
         Args
-        --------
+        ----
         data : any
             Raw data to be encoded usually a dictionary.
 
         Returns
-        ---------
-        bytes:
-             Encoded data.
+        -------
+        `bytes`: Encoded data.
+
         """
         return cls(data).encode()
 
@@ -290,14 +314,14 @@ class Benencoder:
         Encode data provided as an arguement or provided at initialization.
 
         Args
-        --------
+        ----
         val : any, optional
             Data for encoding. Defaults to None.
 
         Returns
-        ---------
-        bytes:
-             encoded data
+        -------
+        `bytes` : encoded data
+
         """
         if val is None:
             val = self.data
@@ -308,82 +332,90 @@ class Benencoder:
         """
         Encode data with bencode protocol.
 
-        args
-        --------
-        bits : bytes
+        Args
+        ----
+        bits : `bytes`
             Bencoded data for decoding.
 
-        returns
-        ---------
-        any:
-             the decoded data.
+        Returns
+        -------
+        any : the decoded data.
+
         """
-        if type(val) == str:
+        if isinstance(val, str):
             return self._encode_str(val)
 
         if hasattr(val, "hex"):
             return self._encode_bytes(val)
 
-        elif type(val) == int:
+        if isinstance(val, int):
             return self._encode_int(val)
 
-        elif type(val) == list:
+        if isinstance(val, list):
             return self._encode_list(val)
 
-        elif type(val) == dict:
+        if isinstance(val, dict):
             return self._encode_dict(val)
 
+        if isinstance(val, tuple):
+            return self._encode_list(list(val))
+
         raise EncodeError(val)
 
-    def _encode_bytes(self, val):
+    @staticmethod
+    def _encode_bytes(val):
+        """Bencode encoding bytes as string literal."""
         size = str(len(val)) + ":"
         return size.encode("utf-8") + val
 
-    def _encode_str(self, txt):
+    @staticmethod
+    def _encode_str(txt):
         """
         Decode string.
 
         Args
-        --------
-        txt (str): string.
+        ----
+        txt : `str`
+            Any string literal.
 
         Returns
-        ---------
-        bytes:
-            Bencoded string.
+        -------
+        `bytes` : Bencoded string.
+
         """
         size = str(len(txt)).encode("utf-8")
         return size + b":" + txt.encode("utf-8")
 
-    def _encode_int(self, i):
+    @staticmethod
+    def _encode_int(num):
         """
         Encode intiger.
 
         Args
-        --------
-        i : int
+        ----
+        num : `int`
             Integer for encoding.
 
         Returns
-        ---------
-        bytes:
-            Bencoded intiger.
+        -------
+        `bytes` : Bencoded intiger.
+
         """
-        return b"i" + str(i).encode("utf-8") + b"e"
+        return b"i" + str(num).encode("utf-8") + b"e"
 
     def _encode_list(self, elems):
         """
         Encode list and its contents.
 
         Args
-        --------
-        elems : list
+        ----
+        elems : `list`
             List of content to be encoded.
 
         Returns
-        ---------
-        bytes:
-            Bencoded data
+        -------
+        `bytes` : Bencoded data
+
         """
         lst = [b"l"]
         for elem in elems:
@@ -398,16 +430,16 @@ class Benencoder:
         Encode keys and values in dictionary.
 
         Args
-        --------
-        dic : dict
+        ----
+        dic : `dict`
             Dictionary of data for encoding.
 
         Returns
-        ---------
-        bytes
-            Bencoded data.
+        -------
+        `bytes` : Bencoded data.
+
         """
         result = b"d"
-        for k, v in dic.items():
-            result += b"".join([self._encode(k), self._encode(v)])
+        for key, val in dic.items():
+            result += b"".join([self._encode(key), self._encode(val)])
         return result + b"e"
